#!/usr/bin/mumps 
;
; Delete any prior global array instances
;
   kill ^df   ; document frequency vector
   kill ^dict ; dictionary vector
   kill ^doc  ; document-term matrix
;
   open 1:"dict.tmp,new" ;* output temp file 
   open 2:"doc.tmp,new"  ;* output temp file
;
   set f=0 ; flag set to false
;
; repeat block until f is true
;
   for  do  quit:f
;
; read in a line from stdin.
; if no more lines, set f true and quit
;
   . use 5 read % if '$test set f=1 quit 
;
; Extract words from line delimited by blank and
; place in array w. Quit when empty string returned.
;
   . for i=1:1 set w(i)=$piece(%," ",i) q:'$len(w(i))
;
; copy docNbr into doc, k is true word count.
; place offset into ^doc(doc)
;
    . set doc=w(2),k=i-1,^doc(doc)=w(1)
;
; For each word in w(), record in ^doc the doc
; number, the word and the offset from the start
; of the doc j.
; Write to unit 2 the document number & word on one 
; line, separated by a blank and, to unit 3.
;
    . for j=3:1:k do 
    .. set ^doc(doc,w(j),j)="" 
    .. use 2 write doc," ",w(j),! 
    .. use 1 write w(j),! 
;
    use 5 
;
    close 1,2 
;
; Execute a shell command. Sort will sort the words in
; dict.tmp alphabetically. Uniq will read the result 
; and write to dict.sorted.tmp one line for each
; unique word preceded by the number of times it 
; occurred.
;
    shell sort < dict.tmp | uniq -c > dict.sorted.tmp 
;
; open the file just created for input
;
    open 1:"dict.sorted.tmp,old" 
    use 1 
;
; Read each line from unit 1. Remove extra blanks.
; Place the word count (piece 1) into ^dict indexed 
; by  the word (piece 2). This constructs the
; dictionary vector.
;
    set f=0 
    for  do  quit:f 
    . read a 
    . if '$test set f=1 quit 
    . set a=$zblanks(a) 
    . set ^dict($piece(a," ",2))=$piece(a," ",1) 

    use 5 
    close 1 
;
; Similar to above. The lines consist of doc number
; and word. The file created will have count,
; doc number and word on each line. The count will
; be the number of times the word appeared in the
; doc.
;
    shell sort < doc.tmp | uniq -c > doc.sorted.tmp 
;
    open 1:"doc.sorted.tmp,old" 
;
; Similar to the above. Read the file just created 
; and store in ^doc for each word, the count.
; Piece 1 is the count, piece 2 is the doc number,
; piece 3 is the word.
; create a transpose matrix of the same data (without
; the word position information). This matrix if the
; term-document matrix. Rows are terms and columns 
; are documents.
;
   use 1 
   set f=0 
   for  do  quit:f 
   . read a 
   . if '$test set f=1 quit 
   . set a=$zblanks(a) 
   . set c=$p(a," ",1) 
   . set a2=$piece(a," ",2),a3=$piece(a," ",3) 
   . set ^doc(a2,a3)=c 
   . set ^index(a3,a2)=c 
;
   use 5 
   close 1 
;
   open 1:"df.tmp,new" 
   use 1 
;
; For each document, write to a temporary file
; each word.
;
   set f1=0,d="" 
   for  do  quit:f1 
   . set d=$order(^doc(d)) 
   . if d="" set f1=1 quit 
   . set f2=0,w="" 
   . for  do  quit:f2 
   .. set w=$order(^doc(d,w)) 
   .. if w="" set f2=1 quit 
   .. write w,! 
;
   close 1 
;
; Sort similar to the above. The output file will be
; lines with a count and a word. The count is will be
; the number of docs the word appeared in.
;
   shell sort < df.tmp | uniq -c > df.sorted.tmp 
;
   open 1:"df.sorted.tmp,old" 
;
; read the file just created and build the doc freq
; vector giving the number of docs each word appears
; in. Piece 2 is the word and piece 1 is the count.
;
   use 1 
   set f=0 
   for  do  quit:f 
   . read a 
   . if '$test set f=1 quit 
   . set a=$zblanks(a) 
   . set ^df($piece(a," ",2))=+a 
;
   close 1 
   use 5 
;
; display the doc-term matrix.
; The modulo operator (#) in the write command
; is used to trigger line feeds after 10 words
; have been written.
;
  use 5 
; 
  set f1=0,d="" 
  for  do  quit:f1 
  . set d=$order(^doc(d)) 
  . if d="" set f1=1 quit 
  . write "Doc ",d,! 
  . set i=0,f2=0,w="" 
  . for  do  quit:f2 
  .. set w=$order(^doc(d,w)) 
  .. if w="" set f2=1 quit 
  .. write w," (",^doc(d,w),") " set i=i+1 write:i#10=7 ! 
  . write !! 
;
  halt

